
from logging import config
import logging.config

from json import load as jsonLoad

from os import sep as osSep

from unittest import TestCase

from pkg_resources import resource_filename


class TestBase(TestCase):

    RESOURCES_PACKAGE_NAME: str = 'tests.testdata'
    RESOURCES_PATH:         str = f'tests{osSep}testdata'

    RESOURCE_ENV_VAR:       str = 'RESOURCEPATH'
    JSON_LOGGING_CONFIG_FILENAME = "testLoggingConfig.json"

    @classmethod
    def setUpLogging(cls):
        """"""
        fqFileName: str = TestBase.retrieveResourcePath(TestBase.JSON_LOGGING_CONFIG_FILENAME)
        with open(fqFileName, 'r') as loggingConfigurationFile:
            configurationDictionary = jsonLoad(loggingConfigurationFile)

        config.dictConfig(configurationDictionary)
        logging.logProcesses = False
        logging.logThreads = False

    @classmethod
    def retrieveResourcePath(cls, bareFileName: str) -> str:

        # Use this method in Python 3.9
        # from importlib_resources import files
        # configFilePath: str  = files('org.pyut.resources').joinpath(Pyut.JSON_LOGGING_CONFIG_FILENAME)

        try:
            fqFileName: str = resource_filename(TestBase.RESOURCES_PACKAGE_NAME, bareFileName)
        except (ValueError, Exception):
            #
            # Maybe we are in an app
            #
            from os import environ
            pathToResources: str = environ.get(f'{TestBase.RESOURCE_ENV_VAR}')
            fqFileName:      str = f'{pathToResources}/{TestBase.RESOURCES_PATH}/{bareFileName}'

        return fqFileName




from typing import Dict

from logging import Logger
from logging import getLogger

from orthogonal.mapping.LayoutGrid import LayoutGrid
from orthogonal.mapping.EmbeddedTypes import Position
from orthogonal.mapping.EmbeddedTypes import Positions
from orthogonal.mapping.EmbeddedTypes import ScreenCoordinates
from orthogonal.mapping.ScreenSize import ScreenSize

IntervalType = Dict[int, int]   # interval, screen coordinate


class EmbeddingToScreen:

    def __init__(self, screenSize: ScreenSize, nodePositions: Positions):

        self.logger:         Logger     = getLogger(__name__)
        self._screenSize:    ScreenSize = screenSize
        self._nodePositions: Positions  = nodePositions

        self._xIntervals: IntervalType = {}
        self._yIntervals: IntervalType = {}

        self._embeddedWidth:  int = 0
        self._embeddedHeight: int = 0

        self._determineGridSize(nodePositions)

        self._layoutGrid: LayoutGrid = LayoutGrid(width=self._embeddedWidth, height=self._embeddedHeight)
        self._layoutGrid.determineZeroZeroNodePosition(nodePositions=nodePositions)

        self._computeXIntervals(self._embeddedWidth - 1)
        self._computeYIntervals(self._embeddedHeight - 1)

    def getScreenPosition(self, nodeName: str) -> ScreenCoordinates:

        gridPos: Position = self._layoutGrid.layoutPositions[nodeName]

        scrX: int = self._xIntervals[gridPos.x]
        scrY: int = self._yIntervals[gridPos.y]
        scrCoordinates: ScreenCoordinates = ScreenCoordinates(scrX, scrY)

        return scrCoordinates

    def _determineGridSize(self, nodePositions: Positions):

        maxX: int = self._findMaxX(nodePositions)
        maxY: int = self._findMaxY(nodePositions)
        minX: int = self._findMinX(nodePositions)
        minY: int = self._findMinY(nodePositions)

        self.logger.info(f'{maxX=} {maxY=} {minX=} {minY=}')

        self._embeddedWidth:  int = abs(minX - maxX) + 1
        self._embeddedHeight: int = abs(minY - maxY) + 1

    def _findMaxX(self, nodePositions: Positions) -> int:

        maxX: int = 0
        for nodeName in nodePositions:
            nodePosition: Position = nodePositions[nodeName]
            if nodePosition.x > maxX:
                maxX = nodePosition.x

        return maxX

    def _findMaxY(self, nodePositions: Positions) -> int:

        maxY: int = 0
        for node in nodePositions:
            nodePosition: Position = nodePositions[node]
            if nodePosition.y > maxY:
                maxY = nodePosition.y

        return maxY

    def _findMinX(self, nodePositions: Positions) -> int:

        minX: int = 0
        for node in nodePositions:
            nodePosition: Position = nodePositions[node]
            if nodePosition.x < minX:
                minX = nodePosition.x

        return minX

    def _findMinY(self, nodePositions: Positions) -> int:

        minY: int = 0
        for node in nodePositions:
            nodePosition: Position = nodePositions[node]
            if nodePosition.y < minY:
                minY = nodePosition.y

        return minY

    def _computeXIntervals(self, maxX: int):
        self._xIntervals = self._computeIntervals(self._screenSize.width, maxX)

    def _computeYIntervals(self, maxY: int):
        self._yIntervals = self._computeIntervals(self._screenSize.height, maxY)

    def _computeIntervals(self, nbrOfPoints: int, maxValue: int) -> IntervalType:

        intervals: IntervalType = {0: 0}

        interval:  int = nbrOfPoints // maxValue
        for x in range(1, maxValue + 1):
            intervals[x] = (x * interval) - 1

        return intervals




from logging import Logger
from logging import getLogger

import networkx as nx

import copy

from orthogonal.topologyShapeMetric.FlowNet import FlowNet
from orthogonal.topologyShapeMetric.Orthogonalization import Orthogonalization
from orthogonal.topologyShapeMetric.Planarization import Planarization


class Compaction:
    """
    Assign minimum lengths to the segments of the edges of the orthogonal representation.
    Never reverse orthogonal in this class.

    """
    def __init__(self, orthogonalization: Orthogonalization):

        self.logger: Logger = getLogger(__name__)

        self._orthogonalization: Orthogonalization = orthogonalization
        if orthogonalization.flow_network.cost == 0:
            self.planar = orthogonalization.planar
            self.flow_dict = orthogonalization.flow_dict
        else:
            self.planar: Planarization = orthogonalization.planar.copy()
            self.flow_dict = copy.deepcopy(orthogonalization.flow_dict)

        self.bend_point_processor()
        self.edge_side = self.face_side_processor()
        self.tidy_rectangle_compaction()
        self.pos = self.layout()

    def bend_point_processor(self):
        """
        Create dummy nodes for bends.
        """
        bends = {}  # left to right
        for he in self.planar.dcel.half_edge_dict.values():
            lf, rf = he.twin.inc, he.inc
            flow = self.flow_dict[lf.id][rf.id][he.id]
            if flow > 0:
                bends[he.id] = flow

        idx = 0
        for he_id, n_bends in bends.items():
            # Q: what if there are bends on both (u, v) and (v, u)?
            # A: Impossible, not a min cost
            he = self.planar.dcel.half_edge_dict[he_id]
            u, v = he.get_points()
            lf_id, rf_id = he.twin.inc.id, he.inc.id

            self.planar.G.remove_edge(u, v)
            self.flow_dict[u][rf_id][u, f'b{idx}'] = self.flow_dict[u][rf_id].pop((u, v))

            for i in range(n_bends):
                cur_node = f'b{idx}'
                pre_node = f'b{idx-1}' if i > 0 else u
                nxt_node = f'b{idx+1}' if i < n_bends - 1 else v
                self.planar.G.add_edge(pre_node, cur_node)
                self.planar.dcel.add_node_between(
                    pre_node, v, cur_node
                )
                self.flow_dict.setdefault(cur_node, {}).setdefault(lf_id, {})[cur_node, pre_node] = 1
                self.flow_dict.setdefault(cur_node, {}).setdefault(rf_id, {})[cur_node, nxt_node] = 3
                idx += 1

            self.flow_dict[v][lf_id][v, f'b{idx-1}'] = self.flow_dict[v][lf_id].pop((v, u))
            self.planar.G.add_edge(f'b{idx-1}', v)

    def face_side_processor(self):
        '''
        Associating edges with face sides.
        '''

        def update_face_edge(edge_side, face, base):
            for he in face.surround_half_edges():
                edge_side[he.id] = (edge_side[he.id] + base) % 4

        edge_side = {}
        for face in self.planar.dcel.face_dict.values():
            # set edges' side in internal faces independently at first
            side = 0
            for he in face.surround_half_edges():
                edge_side[he.id] = side
                end_angle = self.flow_dict[he.succ.ori.id][face.id][he.succ.id]
                if end_angle == 1:
                    # turn right in internal face or turn left in external face
                    side = (side + 1) % 4
                elif end_angle == 3:
                    side = (side + 3) % 4
                elif end_angle == 4:  # a single edge
                    side = (side + 2) % 4

        # update other face's edge side based on ext_face's edge side
        faces_dfs = list(self.planar.dfs_face_order())

        # all faces in dfs order
        has_updated = {faces_dfs[0].id}
        for face in faces_dfs[1:]:
            # at least one twin edge has been set
            for he in face.surround_half_edges():
                lf_id = he.twin.inc.id
                if lf_id in has_updated:  # neighbor face has been updated
                    # the edge that has been updated
                    l_side = edge_side[he.twin.id]
                    r_side = edge_side[he.id]  # side of u, v in face
                    update_face_edge(
                        edge_side, face, (l_side + 2) % 4 - r_side)
                    has_updated.add(face.id)
                    break
        return edge_side

    def tidy_rectangle_compaction(self):
        """
        Doing the compaction of TSM algorithm.
        Compute every edge's length, and store them in self.planar.G.edges[u, v]['len']
        """

        def build_flow(target_side):
            hv_flow = FlowNet()
            for he_id, side in self.edge_side.items():
                if side == target_side:
                    he = self.planar.dcel.half_edge_dict[he_id]
                    lf, rf = he.twin.inc, he.inc
                    lf_id = lf.id
                    rf_id = rf.id if rf.id != self.planar.ext_face.id else 'end'
                    hv_flow.add_edge(lf_id, rf_id, he_id)
            return hv_flow

        def solve(hv_flow, source, sink):
            if not hv_flow:
                return {}
            for node in hv_flow:
                hv_flow.nodes[node]['demand'] = 0
            hv_flow.nodes[source]['demand'] = -2**32
            hv_flow.nodes[sink]['demand'] = 2**32
            for lf_id, rf_id, he_id in hv_flow.edges:
                # what if selfloop?
                hv_flow.edges[lf_id, rf_id, he_id]['weight'] = 1
                hv_flow.edges[lf_id, rf_id, he_id]['lowerbound'] = 1
                hv_flow.edges[lf_id, rf_id, he_id]['capacity'] = 2**32
            hv_flow.add_edge(source, sink, 'extend_edge',
                             weight=0, lowerbound=0, capacity=2**32)

            # self loop avoid inner edge longer than border
            # for u, _ in hv_flow.selfloop_edges():
            #     in_nodes = [v for v, _ in hv_flow.in_edges(u)]
            #     assert in_nodes
            #     delta = sum(hv_flow[v][u]['lowerbound'] for v in in_nodes) - hv_flow[u][u]['count']
            #     if delta < 0:
            #         hv_flow.edges[in_nodes[0]][u]['lowerbound'] += -delta
            return hv_flow.min_cost_flow()

        hor_flow = build_flow(1)  # up -> bottom
        ver_flow = build_flow(0)  # left -> right

        hor_flow_dict = solve(hor_flow, self.planar.ext_face.id, 'end')
        ver_flow_dict = solve(ver_flow, self.planar.ext_face.id, 'end')

        for he in self.planar.dcel.half_edge_dict.values():
            if self.edge_side[he.id] in (0, 1):
                side = self.edge_side[he.id]

                rf = he.inc
                rf_id = 'end' if rf.id == self.planar.ext_face.id else rf.id
                lf_id = he.twin.inc.id

                if side == 0:
                    hv_flow_dict = ver_flow_dict
                elif side == 1:
                    hv_flow_dict = hor_flow_dict

                length = hv_flow_dict[lf_id][rf_id][he.id]
                self.planar.G.edges[he.id]['len'] = length

    def layout(self):
        pos = {}
        for face in self.planar.dfs_face_order():
            for i, u in enumerate(face.nodes_id):
                if not pos:
                    pos[u] = (0, 0)  # initial point
                if u in pos:  # has found a start point
                    new_loop = face.nodes_id[i:] + face.nodes_id[:i]
                    for u, v in zip(new_loop, new_loop[1:]):
                        if v not in pos:
                            side = self.edge_side[u, v]
                            length = self.planar.G.edges[u, v]['len']
                            if side == 1:
                                pos[v] = (pos[u][0] + length, pos[u][1])
                            elif side == 3:
                                pos[v] = (pos[u][0] - length, pos[u][1])
                            elif side == 0:
                                pos[v] = (pos[u][0], pos[u][1] + length)
                            else:  # side == 2
                                pos[v] = (pos[u][0], pos[u][1] - length)
                    break
        return pos

    def check(self):
        for u, v in self.planar.G.edges:
            assert self.pos[u][0] == self.pos[v][0] or self.pos[u][1] == self.pos[v][1]

    def draw(self, **keywords):
        nx.draw(self.planar.G, self.pos, **keywords)




from logging import Logger
from logging import getLogger

from unittest import TestSuite
from unittest import main as unitTestMain

from tests.TestBase import TestBase

# import the class you want to test here
# from org.pyut.template import template


class TestTemplate(TestBase):
    """
    You need to change the name of this class to Test`xxxx`
    Where `xxxx' is the name of the class that you want to test.

    See existing tests for more information.
    """
    clsLogger: Logger = None

    @classmethod
    def setUpClass(cls):
        TestBase.setUpLogging()
        TestTemplate.clsLogger = getLogger(__name__)

    def setUp(self):
        self.logger: Logger = TestTemplate.clsLogger

    def tearDown(self):
        pass

    def testName1(self):
        pass

    def testName2(self):
        """Another test"""
        pass


def suite() -> TestSuite:
    """You need to change the name of the test class here also."""
    import unittest

    testSuite: TestSuite = TestSuite()
    # noinspection PyUnresolvedReferences
    testSuite.addTest(unittest.makeSuite(TestTemplate))

    return testSuite


if __name__ == '__main__':
    unitTestMain()




from orthogonal.doublyConnectedEdgeList.GraphElement import GraphElement


class Hedge(GraphElement):

    def __init__(self, name):
        super().__init__(name)
        self.inc = None # the incident face'
        self.twin = None
        self.ori  = None
        self.pred = None
        self.succ = None

    def get_points(self):
        return self.ori.id, self.twin.ori.id

    def set_all(self, twin, ori, pred, succ, inc):
        self.twin = twin
        self.ori = ori
        self.pred = pred
        self.succ = succ
        self.inc = inc




from typing import Dict
from typing import Tuple

from logging import Logger
from logging import getLogger

import math as m

import networkx as nx

from networkx import Graph

from orthogonal.doublyConnectedEdgeList.Dcel import Dcel
from orthogonal.topologyShapeMetric.OrthogonalException import OrthogonalException

NODE_NAME = str
POSITION  = Tuple[int, int]
POSITIONS = Dict[NODE_NAME, POSITION]


class Planarization:
    """
    This step determines the topology of the drawing which is described by a planar embedding.
    """

    def __init__(self, G: Graph, pos: POSITIONS = None):

        if nx.number_of_selfloops(G) != 0:
            raise OrthogonalException('There can be no self loops in the graph')

        if nx.is_connected(G) is False:
            raise OrthogonalException('The graph or parts of it are not connected.')

        self.logger: Logger = getLogger(__name__)
        if pos is None:
            is_planar, self.embedding = nx.check_planarity(G)
            assert is_planar
            pos = nx.combinatorial_embedding_to_pos(self.embedding)
        else:
            if self.numberOfCrossings(G, pos) != 0:
                raise OrthogonalException('The graph has edges that cross each other')
            self.embedding: nx.PlanarEmbedding = self.convert_pos_to_embedding(G, pos)

        self.G: Graph = G.copy()
        self.pos = pos  # is only used to find the ext_face now.
        self.dcel: Dcel = Dcel(G, self.embedding)
        self.ext_face   = self.get_external_face()

    def copy(self):
        new_planar = self.__new__(self.__class__)
        new_planar.__init__(self.G, self.pos)
        return new_planar

    def get_external_face(self):
        def left_most(G, pos):
            corner_node = min(pos, key=lambda k: (pos[k][0], pos[k][1]))
            other = max(
                G.adj[corner_node], key=lambda node:
                (pos[node][1] - pos[corner_node][1]) /
                m.hypot(
                    pos[node][0] - pos[corner_node][0],
                    pos[node][1] - pos[corner_node][1]
                )
            )  # maximum cosine value
            return sorted([corner_node, other], key=lambda node:
                          (pos[node][1], pos[node][0]))

        if len(self.pos) < 2:
            return list(self.dcel.face_dict.values())[0]
        down, up = left_most(self.G, self.pos)
        return self.dcel.half_edge_dict[up, down].inc

    def dfs_face_order(self):  # dfs dual graph, starts at ext_face
        def dfs_face(face, marked):
            marked.add(face.id)
            yield face
            for neighbor_face in set(face.surround_faces()):
                if neighbor_face.id not in marked:
                    yield from dfs_face(neighbor_face, marked)
        yield from dfs_face(self.ext_face, set())

    def numberOfCrossings(self, G, pos, printIt=True):
        """
        Not accurate, may be equal to actual number or double
        """
        def is_cross(pa, pb, pc, pd):
            def xmul(v1, v2):
                return v1[0] * v2[1] - v1[1] * v2[0]

            def f(pa, pb, p):
                return (pa[1] - pb[1]) * (p[0] - pb[0]) - (p[1] - pb[1]) * (pa[0] - pb[0])

            ca = (pa[0] - pc[0], pa[1] - pc[1])
            cb = (pb[0] - pc[0], pb[1] - pc[1])
            cd = (pd[0] - pc[0], pd[1] - pc[1])
            # return xmul(ca, cd) >= 0 and xmul(cd, cb) >= 0 and f(pa, pb, pc) * f(pa, pb, pd) < 0  # PyCharm Optimization
            return xmul(ca, cd) >= 0 > f(pa, pb, pc) * f(pa, pb, pd) and xmul(cd, cb) >= 0
        count = 0
        for nodeA, nodeB in G.edges:
            for nodeC, nodeD in G.edges:
                if nodeA not in (nodeC, nodeD) and nodeB not in (nodeC, nodeD):
                    if is_cross(pos[nodeA], pos[nodeB], pos[nodeC], pos[nodeD]):
                        count += 1
                        if printIt is True:
                            self.logger.info(nodeA, nodeB, nodeC, nodeD)
        return count

    def convert_pos_to_embedding(self, G: Graph, pos) -> nx.PlanarEmbedding:
        """
            Only straight line in G.
        """
        emd: nx.PlanarEmbedding = nx.PlanarEmbedding()
        for node in G:
            neigh_pos = {
                neigh: (pos[neigh][0]-pos[node][0], pos[neigh][1]-pos[node][1]) for neigh in G[node]
            }
            neighbors_sorted = sorted(G.adj[node], key=lambda v: m.atan2(neigh_pos[v][1], neigh_pos[v][0]))  # counter clockwise
            last = None
            for neigh in neighbors_sorted:
                emd.add_half_edge_ccw(node, neigh, last)
                last = neigh
        emd.check_structure()
        return emd




from typing import Dict


from dataclasses import dataclass

NodeName = str


@dataclass(frozen=True)
class Position:
    """
    This class is immutable;  frozen=true;  Python then creates a good
    __hash__ method
    """

    __slots__ = ['x', 'y']

    x: int
    y: int

    def __le__(self, other: 'Position'):
        """
        Currently this is the only one used
        """
        ans: bool = True

        if self.x > other.x:
            ans = False
        elif self.y > other.y:
            ans = False

        return ans

    def __ge__(self, other: 'Position'):
        """
        Only defined for completeness
        """
        ans: bool = True

        if self.x < other.x:
            ans = False
        elif self.y < other.y:
            ans = False

        return ans


Positions = Dict[NodeName, Position]


@dataclass
class ScreenCoordinates:

    __slots__ = ['x', 'y']

    x: int
    y: int




from typing import Dict
from typing import Set
from typing import cast

from logging import Logger
from logging import getLogger

from copy import deepcopy

from orthogonal.mapping.exceptions.FailedPositioningException import FailedPositioningException
from orthogonal.mapping.exceptions.UnSupportedOperationException import UnSupportedOperationException

from orthogonal.mapping.EmbeddedTypes import NodeName
from orthogonal.mapping.EmbeddedTypes import Position
from orthogonal.mapping.EmbeddedTypes import Positions


GridColumnType = Dict[int, str]
GridType       = Dict[int, GridColumnType]
LayoutPosition = Dict[NodeName, Position]


class LayoutGrid:

    def __init__(self, width: int, height: int):

        self.logger: Logger = getLogger(__name__)

        self._gridWidth  = width
        self._gridHeight = height

        self.layoutPositions:     LayoutPosition = {}
        self._zeroNodePosition:   Position       = cast(Position, None)
        self._grid:               GridType       = self._initializeValidGrid(self._gridWidth, self._gridHeight)

        self.logger.debug(f'{self._grid}')

    @property
    def zeroNodePosition(self) -> Position:
        return self._zeroNodePosition

    @zeroNodePosition.setter
    def zeroNodePosition(self, theNewValue: Position):
        raise UnSupportedOperationException('This is a computed value')

    def determineZeroZeroNodePosition(self, nodePositions: Positions):

        potentialPos: Position = Position(0, 0)
        maxPos:       Position = Position(self._gridWidth - 1, self._gridHeight - 1)

        while potentialPos <= maxPos:
            try:
                self._computeAGridPosition(potentialPos, nodePositions)
                break   # No exception means we found where to put zero zero node
            except FailedPositioningException as fpe:
                self.logger.debug(f'{fpe}')
            potentialPos = self._nextGridPosition(currentGridPosition=potentialPos)

        self._zeroNodePosition: Position = potentialPos

    def _computeAGridPosition(self, theGridPosition: Position, nodePositions: Positions):

        self.logger.debug(f'Check Zero Zero node at: {theGridPosition}')
        inUsePositions: Set = set()

        gridCopy: GridType = deepcopy(self._grid)
        self.layoutPositions = {}   # reset

        for nodeName in nodePositions:

            currentGridPos: Position = nodePositions[nodeName]

            x = theGridPosition.x + currentGridPos.x
            y = theGridPosition.y - currentGridPos.y         # y is always up
            self.logger.debug(f'{currentGridPos=}  grid x,y = ({x},{y})')
            try:
                # aRow = self._grid[x]
                aRow = gridCopy[x]
                # noinspection PyUnusedLocal
                aCell = aRow[y]         # Only used to see if that key will generate a KeyError
                self.logger.debug(f'{currentGridPos=} fits at {x},{y}')
                gridPos: Position = Position(x, y)
                if gridPos in inUsePositions:
                    raise FailedPositioningException(f'grid position {gridPos} in use')
                else:
                    inUsePositions.add(gridPos)
                    aRow[y] = nodeName
                    self.logger.debug(f'{inUsePositions=}')
                    self.layoutPositions[nodeName] = gridPos
            except KeyError:
                self.logger.debug(f'Potential Position: {theGridPosition} failed at computed {x},{y}')
                raise FailedPositioningException(f'Potential Position: {theGridPosition} failed at computed {x},{y}')

        self.logger.info(f'All nodes positioned;  Zero Zero node at: {theGridPosition}')
        self.logger.debug(f'{self.layoutPositions=}')

    def _nextGridPosition(self, currentGridPosition: Position) -> Position:

        nextX: int = currentGridPosition.x + 1
        nextY: int = currentGridPosition.y
        if nextX > self._gridWidth - 1:
            nextX = 0
            nextY: int = currentGridPosition.y + 1

        return Position(nextX, nextY)

    def _initializeValidGrid(self, width: int, height: int) -> Dict[int, GridColumnType]:

        grid: Dict = {}
        for x in range(width):
            col: GridColumnType = {}
            for y in range(height):
                col[y] = f'{x},{y}'  # Just something to size it
            grid[x] = col

        return grid





import collections as coll

from orthogonal.topologyShapeMetric.FlowNet import FlowNet
from orthogonal.topologyShapeMetric.OrthogonalException import OrthogonalException
from orthogonal.topologyShapeMetric.Planarization import Planarization


class Orthogonalization:
    """
    Works on a planar embedding, changes shape of the graph.
    """

    def __init__(self, planar: Planarization):

        # assert max(pair[1] for pair in planar.G.degree) <= 4
        # noinspection PyTypeChecker
        for pair in planar.G.degree:
            if pair[1] > 4:
                raise OrthogonalException('There is a node in this graph with a degree greater than 4')
        assert planar.G.number_of_nodes() > 1

        self.planar = planar

        self.flow_network = self.face_determination()
        self.flow_dict = self.tamassia_orthogonalization()

    def face_determination(self):
        flow_network = FlowNet()

        for vertex in self.planar.dcel.vertex_dict.values():
            flow_network.add_v(vertex.id)

        for face in self.planar.dcel.face_dict.values():
            flow_network.add_f(face.id, len(face), face is self.planar.ext_face)

        for vertex in self.planar.dcel.vertex_dict.values():
            for he in vertex.surround_half_edges():
                flow_network.add_v2f(vertex.id, he.inc.id, he.id)

        for he in self.planar.dcel.half_edge_dict.values():
            flow_network.add_f2f(he.twin.inc.id, he.inc.id, he.id)  # lf -> rf

        return flow_network

    def tamassia_orthogonalization(self):
        return self.flow_network.min_cost_flow()

    def lp_solve(self, weight_of_corner=1, weight_of_sym=0, sym_pairs=None):
        #
        # alert: pulp will automatically transfer node's name into str and replace some special
        # chars into '_', and will throw a error if there are variables' name duplicated.
        #
        import pulp

        prob = pulp.LpProblem()  # minimize

        var_dict = coll.defaultdict(lambda: coll.defaultdict(dict))

        for u, v, he_id in self.flow_network.edges:
            var_dict[u][v][he_id] = pulp.LpVariable(
                f'{u}%{v}%{he_id}',
                self.flow_network[u][v][he_id]['lowerbound'],
                self.flow_network[u][v][he_id]['capacity'],
                pulp.LpInteger
            )

        objs = []
        for he in self.planar.dcel.half_edge_dict.values():
            lf, rf = he.twin.inc.id, he.inc.id
            objs.append(
                self.flow_network[lf][rf][he.id]['weight'] *
                var_dict[lf][rf][he.id]
            )

        # bend points' cost
        if weight_of_corner != 0:
            for v in self.planar.G:
                if self.planar.G.degree(v) == 2:
                    (f1, he1_id), (f2, he2_id) = [(f, key)
                                                  for f, keys in self.flow_network.adj[v].items()
                                                  for key in keys]
                    x = var_dict[v][f1][he1_id]
                    y = var_dict[v][f2][he2_id]
                    p = pulp.LpVariable(x.name + "%temp", None, None, pulp.LpInteger)
                    prob.addConstraint(x - y <= p)
                    prob.addConstraint(y - x <= p)
                    objs.append(weight_of_corner * p)

        # non symmetric cost
        if weight_of_sym != 0:
            if sym_pairs:
                for u, v in sym_pairs:
                    if u != v:
                        faces1 = {
                            face.id for face in self.planar.dcel.vertex_dict[u].surround_faces()}
                        faces2 = {
                            face.id for face in self.planar.dcel.vertex_dict[v].surround_faces()}
                        for f in faces1 & faces2:
                            nodes_id = self.planar.dcel.face_dict[f].nodes_id
                            n = len(nodes_id)
                            u_succ = nodes_id[(nodes_id.index(u) + 1) % n]
                            v_succ = nodes_id[(nodes_id.index(v) + 1) % n]
                            he_u = self.planar.dcel.half_edge_dict[u, u_succ]
                            he_v = self.planar.dcel.half_edge_dict[v, v_succ]

                            x, y = var_dict[u][f][he_u.id], var_dict[v][f][he_v.id]
                            p = pulp.LpVariable(
                                x.name + y.name + "%temp", None, None, pulp.LpInteger)
                            prob.addConstraint(x - y <= p)
                            prob.addConstraint(y - x <= p)
                            objs.append(weight_of_sym * p)

            for v in self.planar.G:
                if self.planar.G.degree(v) == 3:
                    for f, keys in self.flow_network.adj[v].items():
                        if len(keys) == 2:
                            he1_id, he2_id = list(keys)
                            x = var_dict[v][f][he1_id]
                            y = var_dict[v][f][he2_id]
                            p = pulp.LpVariable(
                                x.name + y.name + "%temp", None, None, pulp.LpInteger)
                            prob.addConstraint(x - y <= p)
                            prob.addConstraint(y - x <= p)
                            objs.append(weight_of_sym * p)
        prob += pulp.lpSum(objs), "number of bends in graph"

        for f in self.planar.dcel.face_dict:
            prob += self.flow_network.nodes[f]['demand'] == pulp.lpSum(
                [var_dict[v][f][he_id] for v, _, he_id in self.flow_network.in_edges(f, keys=True)])
        for v in self.planar.G:
            prob += -self.flow_network.nodes[v]['demand'] == pulp.lpSum(
                [var_dict[v][f][he_id] for _, f, he_id in
                 self.flow_network.out_edges(v, keys=True)]
            )

        state = prob.solve()
        if state == 1:  # update flow_dict
            # code here works only when nodes are represented by str, likes '(1, 2)'
            for var in prob.variables():
                if 'temp' not in var.name:
                    lVar = var.name.split('%')
                    if len(lVar) == 3:
                        # u, v, he_id = map(trans, l) # change str to tuple !!!!!!!!!
                        u, v, he_id = [item.replace('_', ' ') for item in lVar]
                        he_id = eval(he_id)
                        self.flow_dict[u][v][he_id] = int(var.varValue)
            return pulp.value(prob.objective)
        else:
            return 2**32

    def number_of_corners(self):
        count_right_angle = 0
        for node in self.planar.G:
            # noinspection PyTypeChecker
            if self.planar.G.degree(node) == 2:
                for f, he_id in [(f, key) for f, keys in self.flow_network.adj[node].items()
                                 for key in keys]:
                    if self.flow_dict[node][f][he_id] == 1:
                        count_right_angle += 1
        return count_right_angle + self.flow_network.cost





from orthogonal.doublyConnectedEdgeList.GraphElement import GraphElement


class Vertex(GraphElement):
    def __init__(self, name):
        super().__init__(name)
        self.inc = None  # 'the first outgoing incident half-edge'
        self.x = None
        self.y = None

    def surround_faces(self):  # clockwise, duplicated
        for he in self.surround_half_edges():
            yield he.inc

    def surround_half_edges(self):  # clockwise
        yield self.inc
        he = self.inc.pred.twin
        while he is not self.inc:
            yield he
            he = he.pred.twin





from typing import List

from logging import Logger
from logging import getLogger

from sys import path as sysPath
from sys import argv as sysArgv

from importlib import import_module

from glob import glob

from unittest import TestResult
from unittest import TextTestRunner
from unittest.suite import TestSuite

from HtmlTestRunner import HTMLTestRunner


class TestAll:
    """
    The class that can run our unit tests in various formats
    """
    NOT_TESTS: List[str] = ['TestAll', 'TestBase', 'TestTemplate']

    VERBOSITY_QUIET:   int = 0  # Print the total numbers of tests executed and the global result
    VERBOSITY_DEFAULT: int = 1  # VERBOSITY_QUIET plus a dot for every successful test or a F for every failure
    VERBOSITY_VERBOSE: int = 2  # Print help string of every test and the result
    VERBOSITY_LOUD:    int = 3  # ??

    def __init__(self):

        self._setupSystemLogging()

        self.logger: Logger = getLogger(__name__)

        self._testSuite: TestSuite = self._getTestSuite()

    def runTextTestRunner(self) -> int:

        status: TestResult = TextTestRunner(verbosity=TestAll.VERBOSITY_QUIET).run(self._testSuite)
        self.logger.info(f'Test Suite Status: {status}')
        if len(status.failures) != 0:
            return 1
        else:
            return 0

    def runHtmlTestRunner(self) -> int:

        runner = HTMLTestRunner(report_name='PyutTestResults', combine_reports=True, add_timestamp=True)
        status = runner.run(self._testSuite)
        if len(status.failures) != 0:
            return 1
        else:
            return 0

    def _setupSystemLogging(self):
        """
        Read the unit test logging configuration file
        """
        from tests.TestBase import TestBase

        TestBase.setUpLogging()

    def _getTestSuite(self) -> TestSuite:
        """

        Returns:
            A suite of all tests in the unit test directory
        """
        modules: List[str] = self.__getTestableModuleNames()
        fSuite: TestSuite = TestSuite()
        for module in modules:
            try:
                fixedName: str = module.replace('/', '.')
                m = import_module(fixedName)
                fSuite.addTest(m.suite())
            except (ValueError, Exception) as e:
                self.logger.error(f'Module import problem with: {module}:  {e}')
        return fSuite

    def __getTestableModuleNames(self) -> List[str]:
        """
        Removes modules that are not unit tests

        Returns:
            A list of module names that we can find in this package
        """

        fModules = glob("tests/Test*.py")
        # remove .py extension
        modules = list(map(lambda x: x[:-3], fModules))
        for doNotTest in TestAll.NOT_TESTS:
            modules.remove(f'tests/{doNotTest}')

        return modules


def main():

    if ".." not in sysPath:
        sysPath.append("..")  # access to the classes to test

    testAll: TestAll = TestAll()
    status: int = 0
    if len(sysArgv) < 2:
        status: int = testAll.runTextTestRunner()
    else:
        for param in sysArgv[1:]:
            if param[:22] == "--produce-html-results":
                print(f'Running HTML Tests')
                status: int = testAll.runHtmlTestRunner()

    return status


if __name__ == "__main__":
    cliStatus: int = main()
    exit(cliStatus)




from typing import Dict
from typing import Tuple

from logging import Logger
from logging import getLogger
from logging import DEBUG

from unittest import TestSuite
from unittest import main as unitTestMain

import networkx as nx
import matplotlib.pyplot as plt

from orthogonal.topologyShapeMetric.Compaction import Compaction
from orthogonal.topologyShapeMetric.Orthogonalization import Orthogonalization
from orthogonal.topologyShapeMetric.Planarization import Planarization

from tests.TestBase import TestBase


class TestGML(TestBase):

    clsLogger: Logger = None

    @classmethod
    def setUpClass(cls):
        TestBase.setUpLogging()
        TestGML.clsLogger = getLogger(__name__)

    def setUp(self):
        self.logger: Logger = TestGML.clsLogger

    def testCase1(self):
        fqFileName: str = TestGML.retrieveResourcePath("case1.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        compact: Compaction = self.generate(G, {node: eval(node) for node in G})

        compact.draw()
        plt.savefig("case1.png")

    def testCase1BiConnected(self):
        fqFileName: str = TestGML.retrieveResourcePath("case1BiConnected.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        compact: Compaction = self.generate(G, {node: eval(node) for node in G})

        compact.draw()
        plt.savefig("case1BiConnected.png")

    def testCase2(self):
        fqFileName: str = TestGML.retrieveResourcePath("case2.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        compact: Compaction = self.generate(G, {node: eval(node) for node in G})

        compact.draw()
        plt.savefig("case2.png")

        if self.logger.level == DEBUG:
            for flowKey in compact.flow_dict.keys():
                self.logger.debug(f'{flowKey=} - {compact.flow_dict[flowKey]=}')

    def testCase2BiConnected(self):
        fqFileName: str = TestGML.retrieveResourcePath("case2BiConnected.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        compact: Compaction = self.generate(G, {node: eval(node) for node in G})
        compact.draw()
        plt.savefig("case2BiConnected.png")

    def testSimple(self):

        fqFileName: str = TestGML.retrieveResourcePath("simple.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        if self.logger.level == DEBUG:
            for node in G:
                self.logger.debug(f'{node=}')

        compact: Compaction = self.generate(G, {node: eval(node) for node in G})

        for flowKey in compact.flow_dict.keys():
            valueDict = compact.flow_dict[flowKey]
            self.logger.debug(f'{flowKey=} - {valueDict}=')
            if self.logger.level == DEBUG:
                for valueKey in valueDict.keys():
                    self.logger.info(f'\t\t{valueKey=} {valueDict[valueKey]=}')

        compact.draw(with_labels=True)
        plt.savefig("simple.png")

    def testTranslationGraphSimple(self):
        fqFileName: str = TestGML.retrieveResourcePath("translationGraphSimple.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        self.logger.debug(f'{G.nodes=}')
        positionDictionary: Dict[str, Tuple] = {}
        for node in G:
            self.logger.debug(f'{node=}')
            x: int = G.nodes[node]['graphics']['x']
            y: int = G.nodes[node]['graphics']['y']
            positionDictionary[node] = (x, y)

        plt.xkcd()
        compact: Compaction = self.generate(G, positionDictionary)

        compact.draw(with_labels=True)
        plt.savefig("translationGraphSimple.png")

    def testTranslationGraphComplex(self):
        fqFileName: str = TestGML.retrieveResourcePath("translationGraphComplex.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        positionDictionary: Dict[str, Tuple] = {}
        for node in G:
            self.logger.debug(f'{node=}')
            x: int = G.nodes[node]['graphics']['x']
            y: int = G.nodes[node]['graphics']['y']
            positionDictionary[node] = (x, y)

        plt.xkcd()
        compact: Compaction = self.generate(G, positionDictionary)
        compact.draw(with_labels=True)

        plt.savefig("translationGraphComplex.png")

    def testTranslationGraphMedium(self):
        fqFileName: str = TestGML.retrieveResourcePath("translationGraphMedium.gml")
        G = nx.Graph(nx.read_gml(fqFileName))
        positionDictionary: Dict[str, Tuple] = {}
        for node in G:
            self.logger.debug(f'{node=}')
            x: int = G.nodes[node]['graphics']['x']
            y: int = G.nodes[node]['graphics']['y']
            positionDictionary[node] = (x, y)

        plt.xkcd()
        compact: Compaction = self.generate(G, positionDictionary)
        compact.draw(with_labels=True)
        plt.savefig("translationGraphMedium.png")

    def generate(self, G, pos=None) -> Compaction:

        planar:     Planarization     = Planarization(G, pos)
        orthogonal: Orthogonalization = Orthogonalization(planar)
        compact:    Compaction        = Compaction(orthogonal)

        return compact


def suite() -> TestSuite:
    import unittest

    testSuite: TestSuite = TestSuite()
    # noinspection PyUnresolvedReferences
    testSuite.addTest(unittest.makeSuite(TestGML))

    return testSuite


if __name__ == '__main__':
    unitTestMain()




import math as m

import networkx as nx


def convert_pos_to_embedding(G, pos):
    """
        Only straight line in G.
    """
    emd = nx.PlanarEmbedding()
    for node in G:
        neigh_pos = {
            neigh: (pos[neigh][0]-pos[node][0], pos[neigh][1]-pos[node][1]) for neigh in G[node]
        }
        neighbors_sorted = sorted(G.adj[node], key=lambda v: m.atan2(neigh_pos[v][1], neigh_pos[v][0]))  # counter clockwise
        last = None
        for neigh in neighbors_sorted:
            emd.add_half_edge_ccw(node, neigh, last)
            last = neigh
    emd.check_structure()
    return emd


def number_of_cross(G, pos, print_it=False):
    """
    Not accurate, may be equal to actual number or double
    This static method is deprecated.  It has been moved to it single usage class as
    Planarization.numberOfCrossings(..)
    """
    def is_cross(pa, pb, pc, pd):
        def xmul(v1, v2):
            return v1[0] * v2[1] - v1[1] * v2[0]

        def f(pa, pb, p):
            return (pa[1] - pb[1]) * (p[0] - pb[0]) - (p[1] - pb[1]) * (pa[0] - pb[0])

        ca = (pa[0] - pc[0], pa[1] - pc[1])
        cb = (pb[0] - pc[0], pb[1] - pc[1])
        cd = (pd[0] - pc[0], pd[1] - pc[1])
        # return xmul(ca, cd) >= 0 and xmul(cd, cb) >= 0 and f(pa, pb, pc) * f(pa, pb, pd) < 0  # PyCharm Optimization
        return xmul(ca, cd) >= 0 > f(pa, pb, pc) * f(pa, pb, pd) and xmul(cd, cb) >= 0
    count = 0
    for a, b in G.edges:
        for c, d in G.edges:
            if a not in (c, d) and b not in (c, d):
                if is_cross(pos[a], pos[b], pos[c], pos[d]):
                    count += 1
                    if print_it:
                        print(a, b, c, d)
    return count



from logging import Logger
from logging import getLogger
from typing import Set

from unittest import TestSuite
from unittest import main as unitTestMain

from tests.TestBase import TestBase


from orthogonal.mapping.EmbeddedTypes import Position


class TestSetOfPositions(TestBase):

    clsLogger: Logger = None

    @classmethod
    def setUpClass(cls):
        TestBase.setUpLogging()
        TestSetOfPositions.clsLogger = getLogger(__name__)

    def setUp(self):
        self.logger: Logger = TestSetOfPositions.clsLogger

    def testInSet(self):

        pos1: Position = Position(4, 4)
        pos2: Position = Position(5, 5)

        aSet: Set = set()
        aSet.add(pos1)
        aSet.add(pos2)

        self.assertTrue(pos1 in aSet, 'But, but I AM in the set')

    def testNotInSet(self):

        pos1: Position = Position(23, 23)
        pos2: Position = Position(42, 42)

        aSet: Set = {pos1, pos2}

        notInSet: Position = Position(7, 7)

        self.assertFalse(notInSet in aSet, 'But, but I am NOT in the set')


def suite() -> TestSuite:
    import unittest

    testSuite: TestSuite = TestSuite()
    # noinspection PyUnresolvedReferences
    testSuite.addTest(unittest.makeSuite(TestSetOfPositions))

    return testSuite


if __name__ == '__main__':
    unitTestMain()




from logging import Logger
from logging import getLogger

from unittest import TestSuite
from unittest import main as unitTestMain

from tests.TestBase import TestBase

from orthogonal.mapping.EmbeddedTypes import ScreenCoordinates
from orthogonal.mapping.ScreenSize import ScreenSize
from orthogonal.mapping.EmbeddedTypes import Positions
from orthogonal.mapping.EmbeddedTypes import Position
from orthogonal.mapping.EmbeddingToScreen import EmbeddingToScreen


class TestEmbeddingToScreen(TestBase):

    clsLogger: Logger = None

    @classmethod
    def setUpClass(cls):
        TestBase.setUpLogging()
        TestEmbeddingToScreen.clsLogger = getLogger(__name__)

    def setUp(self):
        self.logger: Logger = TestEmbeddingToScreen.clsLogger
        self._simplePositions: Positions = {'Node0': Position(0, 0),
                                            'Node5': Position(0, 1),
                                            'Node1': Position(1, 0),
                                            'Node4': Position(1, -1),
                                            'Node3': Position(2, 0),
                                            'Node2': Position(1, 1)
                                            }
        self._complexPositions: Positions = {'Class0': Position(0, 0),
                                             'Class1': Position(0, 1),
                                             'Class2': Position(1, 1),
                                             'Class6': Position(2, 1),
                                             'Class5': Position(1, 2),
                                             'Class7': Position(2, 2),
                                             'Class8': Position(1, 3),
                                             'Class9': Position(0, 2),
                                             'Class4': Position(-1, 1),
                                             'Class3': Position(1, 0)
                                             }
        self._screenSize: ScreenSize = ScreenSize(1000, 1000)

    def testSimple(self):

        ets: EmbeddingToScreen   = EmbeddingToScreen(self._screenSize, self._simplePositions)

        self.assertEqual(3, ets._embeddedWidth)
        self.assertEqual(3, ets._embeddedHeight)

    def testComplex(self):

        ets: EmbeddingToScreen   = EmbeddingToScreen(self._screenSize, self._complexPositions)

        self.assertEqual(4, ets._embeddedWidth)
        self.assertEqual(4, ets._embeddedHeight)

    def testSimpleGetScreenPosition(self):

        ets: EmbeddingToScreen   = EmbeddingToScreen(self._screenSize, self._simplePositions)

        screenPosition: ScreenCoordinates = ets.getScreenPosition('Node1')
        self.assertEqual(499, screenPosition.x, 'X Mapping must be incorrect')
        self.assertEqual(499, screenPosition.y, 'Y Mapping must be incorrect')

        self.logger.info(f'{screenPosition=}')

    def testComplexGetScreenPosition(self):

        ets: EmbeddingToScreen   = EmbeddingToScreen(self._screenSize, self._complexPositions)

        screenPosition: ScreenCoordinates = ets.getScreenPosition('Class4')
        self.assertEqual(0, screenPosition.x, 'X Mapping must be incorrect')
        self.assertEqual(665, screenPosition.y, 'Y Mapping must be incorrect')

        self.logger.debug(f'Class4 - screenPosition: {screenPosition}')

        screenPosition = ets.getScreenPosition('Class6')
        self.assertEqual(998, screenPosition.x, 'X Mapping must be incorrect')
        self.assertEqual(665, screenPosition.y, 'Y Mapping must be incorrect')

        self.logger.debug(f'Class6 - {screenPosition=}')

    def testComputeXIntervals(self):

        ets: EmbeddingToScreen = EmbeddingToScreen(self._screenSize, self._simplePositions)

        ets._computeXIntervals(maxX=2)

        self.logger.debug(f"xIntervals: {ets._xIntervals=}")

        actualXPos: int = ets._xIntervals[1]
        self.assertEqual(499, actualXPos, 'X Computation changed')

    def testComputeYIntervals(self):

        ets: EmbeddingToScreen = EmbeddingToScreen(self._screenSize, self._simplePositions)

        ets._computeYIntervals(maxY=2)

        self.logger.debug(f"{ets._yIntervals=}")
        actualYPos: int = ets._yIntervals[2]
        self.assertEqual(999, actualYPos, 'Y Computation changed')


def suite() -> TestSuite:
    import unittest

    testSuite: TestSuite = TestSuite()
    # noinspection PyUnresolvedReferences
    testSuite.addTest(unittest.makeSuite(TestEmbeddingToScreen))

    return testSuite


if __name__ == '__main__':
    unitTestMain()





import networkx as nx

from orthogonal.doublyConnectedEdgeList.Face import Face
from orthogonal.doublyConnectedEdgeList.Hedge import Hedge
from orthogonal.doublyConnectedEdgeList.Vertex import Vertex


class Dcel:
    def __init__(self, G, embedding):
        assert nx.check_planarity(G)[0]

        self.vertex_dict = {}
        for node in G.nodes:
            self.vertex_dict[node] = Vertex(node)

        self.half_edge_dict = {}
        for u, v in G.edges:
            he1, he2 = Hedge((u, v)), Hedge((v, u))
            self.half_edge_dict[he1.id] = he1
            self.half_edge_dict[he2.id] = he2
            he1.twin = he2
            he1.ori = self.vertex_dict[u]
            self.vertex_dict[u].inc = he1

            he2.twin = he1
            he2.ori = self.vertex_dict[v]
            self.vertex_dict[v].inc = he2

        for he in self.half_edge_dict.values():
            u, v = he.get_points()
            he.succ = self.half_edge_dict[embedding.next_face_half_edge(u, v)]
            he.succ.pred = he

        self.face_dict = {}
        for he in self.half_edge_dict.values():
            if not he.inc:
                face_id = f'f{len(self.face_dict)}'
                face: Face = Face(face_id)
                face.inc = he
                self.face_dict[face_id] = face

                face.nodes_id = embedding.traverse_face(*he.get_points())
                for v1_id, v2_id in zip(face.nodes_id, face.nodes_id[1:]+face.nodes_id[:1]):
                    other = self.half_edge_dict[v1_id, v2_id]
                    assert not other.inc
                    other.inc = face

        if not self.face_dict:
            self.face_dict['f0'] = Face('f0')

    def add_node_between(self, u: 'id', v: 'id', node_name):
        def insert_node(u, v, mid_vertice):
            he = self.half_edge_dict.pop((u, v))
            he1 = Hedge((u, mid_vertice.id))
            he2 = Hedge((mid_vertice.id, v))
            # update half_edge_dict
            self.half_edge_dict[u, mid_vertice.id] = he1
            self.half_edge_dict[mid_vertice.id, v] = he2
            he1.set_all(None, he.ori, he.pred, he2, he.inc)
            he2.set_all(None, mid_vertice, he1, he.succ, he.inc)
            he1.pred.succ = he1
            he2.succ.pred = he2
            # update face
            if he.inc.inc is he:
                he.inc.inc = he1
            he.inc.update_nodes() # not efficient

        # update vertex_dict
        mid_vertice = Vertex(node_name)
        self.vertex_dict[node_name] = mid_vertice
        # insert
        insert_node(u, v, mid_vertice)
        insert_node(v, u, mid_vertice)
        for v1, v2 in ((u, mid_vertice.id), (mid_vertice.id, v)):
            self.half_edge_dict[v1, v2].twin = self.half_edge_dict[v2, v1]
            self.half_edge_dict[v2, v1].twin = self.half_edge_dict[v1, v2]




from logging import Logger
from logging import getLogger

from unittest import TestSuite
from unittest import main as unitTestMain

from orthogonal.mapping.LayoutGrid import GridColumnType
from orthogonal.mapping.LayoutGrid import LayoutGrid

from tests.TestBase import TestBase


from orthogonal.mapping.EmbeddedTypes import Position
from orthogonal.mapping.EmbeddedTypes import Positions


class TestLayoutGrid(TestBase):

    clsLogger: Logger = None

    @classmethod
    def setUpClass(cls):
        TestBase.setUpLogging()
        TestLayoutGrid.clsLogger = getLogger(__name__)

    def setUp(self):
        self.logger: Logger = TestLayoutGrid.clsLogger

    def testInitializeValidGrid(self):

        layoutGrid: LayoutGrid = LayoutGrid(width=9, height=6)

        self.assertTrue(len(layoutGrid._grid) == 9, 'Incorrect width')
        col: GridColumnType = layoutGrid._grid[4]
        self.assertTrue(len(col) == 6, 'Incorrect height')

    def testNextGridPosition(self):
        layoutGrid: LayoutGrid = LayoutGrid(width=9, height=6)

        currentPosition:    Position = Position(3, 3)
        actualNextPosition: Position = layoutGrid._nextGridPosition(currentGridPosition=currentPosition)

        expectedPosition:   Position = Position(4, 3)

        self.assertEqual(expectedPosition, actualNextPosition, 'Oops something is broken')

    def testSimple(self):

        simplePositions: Positions = {'Node0': Position(0, 0),
                                      'Node5': Position(0, 1),
                                      'Node1': Position(1, 0),
                                      'Node4': Position(1, -1),
                                      'Node3': Position(2, 0),
                                      'Node2': Position(1, 1)
                                      }

        layoutGrid: LayoutGrid = LayoutGrid(width=3, height=3)

        layoutGrid.determineZeroZeroNodePosition(nodePositions=simplePositions)

        expectedZeroZeroPosition: Position = Position(0, 1)
        self.assertEqual(expectedZeroZeroPosition, layoutGrid.zeroNodePosition)

    def testComplex(self):
        complexPositions: Positions = {'Class0': Position(0, 0),
                                       'Class1': Position(0, 1),
                                       'Class2': Position(1, 1),
                                       'Class6': Position(2, 1),
                                       'Class5': Position(1, 2),
                                       'Class7': Position(2, 2),
                                       'Class8': Position(1, 3),
                                       'Class9': Position(0, 2),
                                       'Class4': Position(-1, 1),
                                       'Class3': Position(1, 0),
                                       }
        layoutGrid: LayoutGrid = LayoutGrid(width=4, height=4)

        layoutGrid.determineZeroZeroNodePosition(nodePositions=complexPositions)

        expectedZeroZeroPosition: Position = Position(1, 3)
        self.assertEqual(expectedZeroZeroPosition, layoutGrid.zeroNodePosition)

    # def testLayoutFail(self):
    #
    #     complexPositions: Positions = {'Class0': Position(0, 0),
    #                                    'Class1': Position(0, 1),
    #                                    'Class4': Position(0, 2),
    #                                    'Class2': Position(-1, 1),
    #                                    'Class5': Position(-1, 2),
    #                                    'Class9': Position(0, 2),      # This was a bug in the layout engine
    #                                    'Class8': Position(-1, 3),
    #                                    'Class7': Position(-2, 2),
    #                                    'Class6': Position(-1, 0),
    #                                    'Class3': Position(1, 0)
    #                                    }
    #
    #     layoutGrid: LayoutGrid = LayoutGrid(width=4, height=4, nodePositions=complexPositions)
    #
    #     expectedZeroZeroPosition: Position = Position(2, 3)
    #     self.assertEqual(expectedZeroZeroPosition, layoutGrid.zeroNodePosition)


def suite() -> TestSuite:
    import unittest

    testSuite: TestSuite = TestSuite()
    # noinspection PyUnresolvedReferences
    testSuite.addTest(unittest.makeSuite(TestLayoutGrid))

    return testSuite


if __name__ == '__main__':
    unitTestMain()



import pathlib
from setuptools import setup
from setuptools import find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name="orthogonal",
    version="1.1.5",
    description="Orthogonal Graph Layout for Python 3",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/hasii2011/OrthogonalDrawing",
    packages=find_packages(),
    include_package_data=True
)




import collections as coll

import networkx as nx


class FlowNet(nx.MultiDiGraph):
    def add_v2f(self, v, f, key):
        self.add_edge(v, f, key=key, lowerbound=1, capacity=4, weight=0)

    def add_f2f(self, f1, f2, key):
        # if not self.has_edge(f1, f2):
        self.add_edge(f1, f2, key=key, lowerbound=0, capacity=2**32, weight=1)

    def add_v(self, v):
        self.add_node(v, demand=-4)     # the total degree around a node is 2pi

    def add_f(self, f, degree, is_external):
        # the degree of a face is the length of the cycle bounding the face.
        self.add_node(f, demand=(2 * degree + 4) if is_external else (2 * degree - 4))

    def min_cost_flow(self):
        def get_demand(flow_dict, node):
            in_flow = sum(flow_dict[u][v][key]
                          for u, v, key in self.in_edges(node, keys=True))
            out_flow = sum(flow_dict[u][v][key]
                           for u, v, key in self.out_edges(node, keys=True))
            return in_flow - out_flow

        def split(multi_flowG):
            base_dict = coll.defaultdict(lambda: coll.defaultdict(dict))
            new_mdg = nx.MultiDiGraph()

            for u, v, key in multi_flowG.edges:
                lowerbound = multi_flowG[u][v][key]['lowerbound']
                base_dict[u][v][key] = lowerbound
                new_mdg.add_edge(u, v, key,
                                          capacity=multi_flowG[u][v][key]['capacity'] - lowerbound,
                                          weight=multi_flowG[u][v][key]['weight'],
                                          )
            for node in multi_flowG:
                new_mdg.nodes[node]['demand'] =  \
                    multi_flowG.nodes[node]['demand'] - \
                    get_demand(base_dict, node)
            return base_dict, new_mdg

        base_dict, new_mdg = split(self)
        flow_dict = nx.min_cost_flow(new_mdg)
        for u, v, key in self.edges:
            flow_dict[u][v][key] += base_dict[u][v][key]

        self.cost = self.cost_of_flow(flow_dict)
        return flow_dict

    def cost_of_flow(self, flow_dict):
        cost = 0
        for u, v, key in self.edges:
            cost += flow_dict[u][v][key] * self[u][v][key]['weight']
        return cost



import networkx as nx
from orthogonal.topologyShapeMetric import TSM
import unittest


def generate(G, pos=None):
    planar = TSM.Planarization(G, pos)
    orthogonal = TSM.Orthogonalization(planar)
    compact = TSM.Compaction(orthogonal)


class TestGML(unittest.TestCase):
    def test_01(self):
        G = nx.Graph(nx.read_gml("testdata/case1.gml"))
        generate(G, {node: eval(node) for node in G})

    def test_02(self):
        G = nx.Graph(nx.read_gml("testdata/case1BiConnected.gml"))
        generate(G, {node: eval(node) for node in G})

    def test_03(self):
        G = nx.Graph(nx.read_gml("testdata/case2.gml"))
        generate(G, {node: eval(node) for node in G})

    def test_04(self):
        G = nx.Graph(nx.read_gml("testdata/case2BiConnected.gml"))
        generate(G, {node: eval(node) for node in G})


class TestGrid(unittest.TestCase):

    def _test_grid(self, i, j):
        G = nx.grid_2d_graph(i, j)
        generate(G, pos={node: node for node in G})
        generate(G)

    def test_01(self):
        self._test_grid(1, 2)

    def test_02(self):
        self._test_grid(2, 1)

    def test_03(self):
        self._test_grid(1, 5)

    def test_04(self):
        self._test_grid(5, 1)

    def test_05(self):
        self._test_grid(5, 5)

    def test_06(self):
        self._test_grid(3, 3)

    def test_07(self):
        self._test_grid(9, 9)


if __name__ == '__main__':
    res = unittest.main(verbosity=3, exit=False)




from orthogonal.doublyConnectedEdgeList.GraphElement import GraphElement


class Face(GraphElement):
    def __init__(self, name):
        super().__init__(name)
        self.inc = None  # the first half-edge incident to the face from left
        self.nodes_id = []

    def __len__(self):
        return len(self.nodes_id)

    def __repr__(self):
        return f'FaceView{repr(self.nodes_id)}'

    def update_nodes(self):
        self.nodes_id = [vertex.id for vertex in self.surround_vertices()]

    def surround_faces(self):  # clockwise, duplicated!!
        for he in self.surround_half_edges():
            yield he.twin.inc

    def surround_half_edges(self):  # clockwise
        yield self.inc
        he = self.inc.succ
        while he is not self.inc:
            yield he
            he = he.succ

    def surround_vertices(self):
        for he in self.surround_half_edges():
            yield he.ori
